@@ -48,29 +48,39 @@ class CatBoostAlgorithm(BaseAlgorithm):
             loss_function = self.params.get("loss_function", "RMSE")
             Algo = CatBoostRegressor
 
-        self.learner_params = {
+        cat_params = {
+            "iterations": self.params.get("num_boost_round", self.rounds),
             "learning_rate": self.params.get("learning_rate", 0.1),
             "depth": self.params.get("depth", 3),
             "rsm": self.params.get("rsm", 1.0),
-            "random_seed": self.params.get("seed", 1),
+            "l2_leaf_reg": self.params.get("l2_leaf_reg", 3.0),
+            "random_strength": self.params.get("random_strength", 1.0),
             "loss_function": loss_function,
+            "eval_metric": self.params.get("eval_metric", loss_function),
+            "custom_metric": self.params.get("eval_metric", loss_function),
+            "thread_count": self.params.get("n_jobs", -1),
+            "verbose": False,
+            "allow_writing_files": False,
+            "random_seed": self.params.get("seed", 1),
         }
-        self.model = Algo(
-            iterations=self.rounds,
-            learning_rate=self.learner_params["learning_rate"],
-            depth=self.learner_params["depth"],
-            rsm=self.learner_params["rsm"],
-            loss_function=self.learner_params["loss_function"],
-            thread_count=self.params.get("n_jobs", -1),
-            verbose=False,
-            allow_writing_files=False,
-        )
+
+        for extra_param in [
+            "min_data_in_leaf",
+            "bootstrap_type",
+            "bagging_temperature",
+            "subsample",
+            "border_count",
+        ]:
+            if extra_param in self.params:
+                cat_params[extra_param] = self.params[extra_param]
+
+        self.model = Algo(**cat_params)
         self.cat_features = None
         self.best_ntree_limit = 0
 
         logger.debug("CatBoostAlgorithm.__init__")
 
-    def _assess_iterations(self, X, y, eval_set, max_time=None):
+    def _assess_iterations(self, X, y, sample_weight, eval_set, max_time=None):
         if max_time is None:
             max_time = 3600
         try:
@@ -80,6 +90,7 @@ class CatBoostAlgorithm(BaseAlgorithm):
             model.fit(
                 X,
                 y,
+                sample_weight=sample_weight,
                 cat_features=self.cat_features,
                 init_model=None if self.model.tree_count_ is None else self.model,
                 eval_set=eval_set,
@@ -125,9 +136,16 @@ class CatBoostAlgorithm(BaseAlgorithm):
                 weight=sample_weight_validation,
             )
 
-        # disable for now ...
-        model_init, new_iterations = self._assess_iterations(X, y, eval_set, max_time)
-        self.model.set_params(iterations=new_iterations)
+        if self.params.get("num_boost_round") is None:
+            model_init, new_iterations = self._assess_iterations(
+                X, y, sample_weight, eval_set, max_time
+            )
+            self.model.set_params(iterations=new_iterations)
+        else:
+            model_init = None
+            self.model.set_params(iterations=self.params.get("num_boost_round"))
+            self.early_stopping_rounds = self.params.get("early_stopping_rounds", 50)
+
         self.model.fit(
             X,
             y,
@@ -138,10 +156,15 @@ class CatBoostAlgorithm(BaseAlgorithm):
             early_stopping_rounds=self.early_stopping_rounds,
             verbose_eval=False,
         )
+        
         if self.model.best_iteration_ is not None:
-            self.best_ntree_limit = (
-                self.model.best_iteration_ + model_init.tree_count_ + 1
-            )
+            if model_init is not None:
+                self.best_ntree_limit = (
+                    self.model.best_iteration_ + model_init.tree_count_ + 1
+                )
+            else:
+                self.best_ntree_limit = self.model.best_iteration_ + 1
+
         else:
             # just take all the trees
             # the warm-up trees are already included
@@ -149,7 +172,6 @@ class CatBoostAlgorithm(BaseAlgorithm):
             self.best_ntree_limit = self.model.tree_count_
 
         if log_to_file is not None:
-
             metric_name = list(self.model.evals_result_["learn"].keys())[0]
             train_scores = self.model.evals_result_["learn"][metric_name]
             validation_scores = self.model.evals_result_["validation"][metric_name]
