@@ -63,15 +63,23 @@ class XgbAlgorithm(BaseAlgorithm):
             "seed": self.params.get("seed", 1),
         }
 
+        if "lambda" in self.params:
+            self.learner_params["lambda"] = self.params["lambda"]
+        if "alpha" in self.params:
+            self.learner_params["alpha"] = self.params["alpha"]
+
         # check https://github.com/dmlc/xgboost/issues/5637
         if self.learner_params["seed"] > 2147483647:
             self.learner_params["seed"] = self.learner_params["seed"] % 2147483647
         if "num_class" in self.params:  # multiclass classification
             self.learner_params["num_class"] = self.params.get("num_class")
 
+        if "max_rounds" in self.params:
+            self.boosting_rounds = self.params["max_rounds"]
+
         self.best_ntree_limit = 0
         logger.debug("XgbLearner __init__")
-
+        
     def get_boosting_rounds(self, dtrain, evals, esr, max_time):
         if max_time is None:
             return self.boosting_rounds
