@@ -98,6 +98,8 @@ class BaseAutoML(BaseEstimator, ABC):
         self._kmeans_features = None
         self._mix_encoding = None
         self._max_single_prediction_time = None
+        self._optuna_time_budget = None
+        self._optuna_init_params = {}
         self._n_jobs = -1
 
     def _get_tuner_params(
@@ -412,7 +414,7 @@ class BaseAutoML(BaseEstimator, ABC):
 
         org_index = X.index.copy()
         X.reset_index(drop=True, inplace=True)
-        X_stacked = pd.concat(all_oofs + [X], axis=1)
+        X_stacked = pd.concat([X] + all_oofs, axis=1)
 
         X_stacked.index = org_index.copy()
         X.index = org_index.copy()
@@ -732,11 +734,11 @@ class BaseAutoML(BaseEstimator, ABC):
             self.verbose_print(
                 "Neural Network algorithm was disabled because it doesn't support n_jobs parameter."
             )
-        if "Linear" in self._algorithms and not (self.n_rows_in_ < 10000 and self.n_features_in_ < 1000):
+        if "Linear" in self._algorithms and not (
+            self.n_rows_in_ < 10000 and self.n_features_in_ < 1000
+        ):
             self._algorithms.remove("Linear")
-            self.verbose_print(
-                "Linear algorithm was disabled."
-            )
+            self.verbose_print("Linear algorithm was disabled.")
 
         # remove algorithms in the case of multiclass
         # and too many classes and columns
@@ -807,9 +809,13 @@ class BaseAutoML(BaseEstimator, ABC):
             k_folds = 5
             if folds_cnt >= 15:
                 k_folds = 10
-            # too small dataset to run 10-fold CV
-            if k_folds == 10 and self.n_rows_in_ <= 1000:
-                k_folds = 5
+            # too small dataset for stacking
+            if self.n_rows_in_ < 500:
+                self._stack_models = False
+                self.verbose_print(
+                    "*** Disable stacking for small dataset (nrows < 500)"
+                )
+
             self._validation_strategy["validation_type"] = "kfold"
             del self._validation_strategy["train_ratio"]
             self._validation_strategy["k_folds"] = k_folds
@@ -900,6 +906,8 @@ class BaseAutoML(BaseEstimator, ABC):
         self._kmeans_features = self._get_kmeans_features()
         self._mix_encoding = self._get_mix_encoding()
         self._max_single_prediction_time = self._get_max_single_prediction_time()
+        self._optuna_time_budget = self._get_optuna_time_budget()
+        self._optuna_init_params = self._get_optuna_init_params()
         self._n_jobs = self._get_n_jobs()
         self._random_state = self._get_random_state()
 
@@ -962,6 +970,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 self._boost_on_errors,
                 self._kmeans_features,
                 self._mix_encoding,
+                self._optuna_time_budget,
+                self._optuna_init_params,
                 self._n_jobs,
                 self._random_state,
             )
@@ -1413,6 +1423,14 @@ class BaseAutoML(BaseEstimator, ABC):
                     "Neural Network",
                     "Nearest Neighbors",
                 ]
+            if self._get_mode() == "Optuna":
+                return [
+                    "Random Forest",
+                    "Extra Trees",
+                    "LightGBM",
+                    "Xgboost",
+                    "CatBoost",
+                ]
         else:
             return deepcopy(self.algorithms)
 
@@ -1425,7 +1443,7 @@ class BaseAutoML(BaseEstimator, ABC):
         """ Gets the current stack_models"""
         self._validate_stack_models()
         if self.stack_models == "auto":
-            return True if self.mode == "Compete" else False
+            return True if self.mode in ["Compete", "Optuna"] else False
         else:
             return deepcopy(self.stack_models)
 
@@ -1461,7 +1479,7 @@ class BaseAutoML(BaseEstimator, ABC):
                     "shuffle": True,
                     "stratify": True,
                 }
-            elif self._get_mode() == "Compete":
+            elif self._get_mode() in ["Compete", "Optuna"]:
                 strat = {
                     "validation_type": "kfold",
                     "k_folds": 10,
@@ -1496,6 +1514,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 return 1
             if self._get_mode() == "Compete":
                 return 0
+            if self._get_mode() == "Optuna":
+                return 0
         else:
             return deepcopy(self.explain_level)
 
@@ -1508,6 +1528,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 return True
             if self._get_mode() == "Compete":
                 return True
+            if self._get_mode() == "Optuna":
+                return True
         else:
             return deepcopy(self.golden_features)
 
@@ -1521,6 +1543,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 return True
             if self._get_mode() == "Compete":
                 return True
+            if self._get_mode() == "Optuna":
+                return True
         else:
             return deepcopy(self.features_selection)
 
@@ -1534,6 +1558,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 return 5
             if self._get_mode() == "Compete":
                 return 10
+            if self._get_mode() == "Optuna":
+                return 1  # just 1, because it will be tuned by Optuna
         else:
             return deepcopy(self.start_random_models)
 
@@ -1547,6 +1573,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 return 2
             if self._get_mode() == "Compete":
                 return 2
+            if self._get_mode() == "Optuna":
+                return 0  # all tuning is done in Optuna
         else:
             return deepcopy(self.hill_climbing_steps)
 
@@ -1560,6 +1588,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 return 2
             if self._get_mode() == "Compete":
                 return 3
+            if self._get_mode() == "Optuna":
+                return 0
         else:
             return deepcopy(self.top_models_to_improve)
 
@@ -1573,6 +1603,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 return False
             if self._get_mode() == "Compete":
                 return True
+            if self._get_mode() == "Optuna":
+                return True
         else:
             return deepcopy(self.boost_on_errors)
 
@@ -1586,6 +1618,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 return False
             if self._get_mode() == "Compete":
                 return True
+            if self._get_mode() == "Optuna":
+                return True
         else:
             return deepcopy(self.kmeans_features)
 
@@ -1599,6 +1633,8 @@ class BaseAutoML(BaseEstimator, ABC):
                 return False
             if self._get_mode() == "Compete":
                 return True
+            if self._get_mode() == "Optuna":
+                return True
         else:
             return deepcopy(self.mix_encoding)
 
@@ -1612,6 +1648,28 @@ class BaseAutoML(BaseEstimator, ABC):
         else:
             return deepcopy(self.max_single_prediction_time)
 
+    def _get_optuna_time_budget(self):
+        """ Gets the current optuna_time_budget"""
+        self._validate_optuna_time_budget()
+
+        if self.optuna_time_budget is None:
+            if self._get_mode() == "Optuna":
+                return 3600
+            return None
+        else:
+            if self._get_mode() != "Optuna":
+                # use only for mode Optuna
+                return None
+            return deepcopy(self.optuna_time_budget)
+
+    def _get_optuna_init_params(self):
+        """ Gets the current optuna_init_params"""
+        self._validate_optuna_init_params()
+        if self._get_mode() != "Optuna":
+            # use only for mode Optuna
+            return {}
+        return deepcopy(self.optuna_init_params)
+
     def _get_n_jobs(self):
         """ Gets the current n_jobs"""
         self._validate_n_jobs()
@@ -1624,7 +1682,7 @@ class BaseAutoML(BaseEstimator, ABC):
 
     def _validate_mode(self):
         """ Validates mode parameter"""
-        valid_modes = ["Explain", "Perform", "Compete"]
+        valid_modes = ["Explain", "Perform", "Compete", "Optuna"]
         if self.mode not in valid_modes:
             raise ValueError(
                 f"Expected 'mode' to be {' or '.join(valid_modes)}, got '{self.mode}'"
@@ -1822,6 +1880,21 @@ class BaseAutoML(BaseEstimator, ABC):
             self.max_single_prediction_time, "max_single_prediction_time"
         )
 
+    def _validate_optuna_time_budget(self):
+        """ Validates optuna_time_budget parameter"""
+        if self.optuna_time_budget is None:
+            return
+        check_greater_than_zero_integer(self.optuna_time_budget, "optuna_time_budget")
+
+    def _validate_optuna_init_params(self):
+        """ Validates optuna_init_params parameter"""
+        if self.optuna_init_params is None:
+            return
+        if type(self.optuna_init_params) is not dict:
+            raise ValueError(
+                f"Expected 'optuna_init_params' to be a dict, got '{type(self.optuna_init_params)}'"
+            )
+
     def _validate_n_jobs(self):
         """ Validates mix_encoding parameter"""
         check_integer(self.n_jobs, "n_jobs")
@@ -1961,9 +2034,8 @@ h3 {
         if main_readme_html is not None:
             return IFrame(main_readme_html, width, height)
 
-
     def _need_retrain(self, X, y, sample_weight, decrease):
-        
+
         metric = self._best_model.get_metric()
 
         X, y, sample_weight = ExcludeRowsMissingTarget.transform(
@@ -1971,7 +2043,7 @@ h3 {
         )
 
         if self._ml_task == BINARY_CLASSIFICATION:
-            prediction = self._predict_proba(X)[:,1]
+            prediction = self._predict_proba(X)[:, 1]
         if self._ml_task == MULTICLASS_CLASSIFICATION:
             prediction = self._predict_proba(X)
         else:
@@ -1979,17 +2051,19 @@ h3 {
 
         sign = -1.0 if Metric.optimize_negative(metric.name) else 1.0
 
-        
         new_score = metric(y, prediction, sample_weight)
         old_score = self._best_model.get_final_loss()
 
         change = np.abs((old_score - new_score) / old_score)
 
-        # always minimize the score 
+        # always minimize the score
         if new_score > old_score:
-            self.verbose_print(f"Model performance decreased by {np.round(change*100.0,2)}%")
+            self.verbose_print(
+                f"Model performance decreased by {np.round(change*100.0,2)}%"
+            )
             return change > decrease
         else:
-            self.verbose_print(f"Model performance increased by {np.round(change*100.0,2)}%")
-            return False    
-        
\ No newline at end of file
+            self.verbose_print(
+                f"Model performance increased by {np.round(change*100.0,2)}%"
+            )
+            return False
